SystemOrganization addCategory: #'Pier-Squeak-Persistency'!

PRPersistencyTest subclass: #PRImagePersistencyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistencyTest class methodsFor: 'testing' stamp: 'lr 3/30/2007 22:22'!
isAbstract
	^ false! !

!PRImagePersistencyTest methodsFor: 'private' stamp: 'lr 3/30/2007 22:22'!
actualClass
	^ PRImagePersistency! !

PRHistoryPersistency subclass: #PRImagePersistency
	instanceVariableNames: 'duration process maxImages namePrefix lastSnapshot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Squeak-Persistency'!

!PRImagePersistency commentStamp: 'lr 6/22/2008 17:53' prior: 0!
I am a persistency strategy regularly saving the whole image. Do not use me when developing, as this might corrupt the changes file.!

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 9/23/2008 08:14'!
descriptionDuration
	^ MADurationDescription new
		comment: 'The minimum interval between two snapshots.';
		accessor: #duration;
		label: 'Duration';
		default: 1 hour;
		priority: 130;
		beRequired;
		yourself! !

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 9/19/2008 11:24'!
descriptionIsScheduled
	^ MABooleanDescription new
		label: 'Snapshot scheduled';
		accessor: #isScheduled;
		group: 'Information';
		priority: 210;
		beReadonly;
		yourself! !

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 9/19/2008 11:24'!
descriptionLastSnapshot
	^ MATimeStampDescription new
		accessor: #lastSnapshot;
		label: 'Last Snapshot';
		group: 'Information';
		priority: 200;
		beReadonly;
		yourself! !

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 9/23/2008 08:14'!
descriptionMaxImages
	^ MANumberDescription new
		comment: 'The maximum number of backup copies to keep';
		accessor: #maxImages;
		label: 'Max Images';
		priority: 110;
		beRequired;
		default: 3;
		beInteger;
		yourself! !

!PRImagePersistency class methodsFor: 'accessing-descriptions' stamp: 'lr 1/23/2009 13:56'!
descriptionNamePrefix
	^ MAStringDescription new
		comment: 'Defines the prefix for image-names.';
		accessor: #namePrefix;
		default: 'pier_backup_';
		label: 'Name Prefix';
		priority: 120;
		beRequired;
		yourself! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 9/23/2008 08:12'!
cleanup
	| images |
	maxImages isNil ifTrue: [ ^ self ].
	images := FileDirectory default fileNames select: [ :each | 
		(each endsWith: FileDirectory imageSuffix) 
			and: [ each beginsWith: namePrefix ] ].
	images size <= maxImages ifTrue: [ ^ self ].
	images := (images asSortedCollection allButLast: maxImages)
		collect: [ :each | each copyUpToLast: $. ].
	images do: [ :each |
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory imageSuffix)
			ifAbsent: [ ].
		FileDirectory default 
			deleteFileNamed: (each , '.' , FileDirectory changeSuffix)
			ifAbsent: [ ] ]! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 9/29/2006 08:39'!
duration
	^ duration! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:00'!
duration: aDuration
	"The minimum interval between two snapshots."

	duration := aDuration! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 11/20/2007 16:00'!
execute: aCommand
	self mutex critical: [
		self apply: aCommand.
		aCommand isLogged ifTrue: [
			self add: aCommand context.
			self log: aCommand context ] ].
	self start! !

!PRImagePersistency methodsFor: 'testing' stamp: 'lr 9/19/2008 11:25'!
isScheduled
	^ process notNil and: [ process isTerminated not ]! !

!PRImagePersistency methodsFor: 'accessing-readonly' stamp: 'lr 9/19/2008 11:20'!
lastSnapshot
	^ lastSnapshot! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 1/17/2007 10:37'!
log: aContext
	| stream |
	stream := FileDirectory default
		fileNamed: 'transactions.txt'.
	[ self log: aContext on: stream setToEnd ]
		ensure: [ stream close ]! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 2/12/2007 19:07'!
log: aContext on: aStream
	aStream nextPutAll: ' == '; nextPutAll: aContext command class label; nextPutAll: ' == '; cr.
	aStream nextPutAll: ' -- context -- '; cr.
	aContext description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext
				readUsing: each)); cr ].
	aStream nextPutAll: ' -- command -- '; cr.
	aContext command description do: [ :each |
		aStream
			nextPutAll: each label; nextPutAll: ': ';
			nextPutAll: (each toString: (aContext command
				readUsing: each)); cr ].
	aStream cr; cr! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 17:54'!
maxImages
	^ maxImages! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 17:55'!
maxImages: anInteger
	"The maximum number of backup copies to keep."

	maxImages := anInteger! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:06'!
namePrefix
	^ namePrefix! !

!PRImagePersistency methodsFor: 'accessing' stamp: 'lr 6/22/2008 18:06'!
namePrefix: aString
	"Defines the prefix for image-names."

	namePrefix := aString! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 6/22/2008 18:44'!
process
	(Delay forDuration: self duration)
		wait.
	process := nil.
	self snapshot; cleanup! !

!PRImagePersistency methodsFor: 'private' stamp: 'lr 1/23/2009 13:58'!
saveImageAndBackupAs: aString
	| image |
	image := SmalltalkImage current.
	image closeSourceFiles.
	FileDirectory default
		copyFileWithoutOverwriteConfirmationNamed: SmalltalkImage current imageName 
		toFileNamed: (FileDirectory default pathName , FileDirectory slash , aString , '.' , FileDirectory imageSuffix).
	FileDirectory default
		copyFileWithoutOverwriteConfirmationNamed: SmalltalkImage current changesName 
		toFileNamed: (FileDirectory default pathName , FileDirectory slash , aString , '.' , FileDirectory changeSuffix).	
	Smalltalk setMacFileInfoOn: (FileDirectory default pathName , FileDirectory slash , aString , '.' , FileDirectory changeSuffix).
	image openSourceFiles; saveImageSegments; snapshot: true andQuit: false! !

!PRImagePersistency methodsFor: 'initialization' stamp: 'lr 6/22/2008 18:00'!
setKernel: aKernel
	super setKernel: aKernel.
	self stop! !

!PRImagePersistency methodsFor: 'actions' stamp: 'lr 1/23/2009 13:55'!
snapshot
	"Save the whole image."

	| timestamp |
	timestamp := DateAndTime now.
	self saveImageAndBackupAs: namePrefix , timestamp asSortableString.
	lastSnapshot := timestamp! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 9/19/2008 11:25'!
start
	self isScheduled
		ifTrue: [ ^ self ].
	process := [ self process ]
		forkAt: 30 
		named: 'pier'! !

!PRImagePersistency methodsFor: 'process' stamp: 'lr 9/19/2008 11:25'!
stop
	self isScheduled
		ifTrue: [ process terminate ].
	process := nil! !

!DateAndTime methodsFor: '*pier-squeak-persistency' stamp: 'lr 9/23/2008 08:13'!
asSortableString
	"A small interpreter to get a date exactly the way required."

	| code |
	code := #( year 4 $- monthIndex 2 $- dayOfMonth 2 $_ hour 2 $- minute 2 $- second 2 ) readStream.
	^ String streamContents: [ :stream |
		[ code atEnd ] whileFalse: [
			stream nextPutAll: ((self perform: code next)
				printPaddedWith: $0 to: code next).
			code next
				ifNotNilDo: [ :separator | stream nextPut: separator ] ] ]! !
